@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2007 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: PcgDun.cpp,v 1.56 2007/12/25 06:13:08 bowkenken Exp $
+* $Id: PcgDun.cpp,v 1.62 2008/04/03 10:26:45 bowkenken Exp $
 ***************************************************************/
 
 #include	"gmain.h"
@@ -48,7 +48,7 @@
 #include	"amenu.h"
 //#include	"request.h"
 #include	"gfile.h"
-//#include	"msg.h"
+#include	"msg.h"
 //#include	"ver.h"
 #include	"gmain-prot.h"
 #include	"misc-prot.h"
@@ -73,7 +73,7 @@
 //#include	"amenu-prot.h"
 //#include	"request-prot.h"
 #include	"gfile-prot.h"
-//#include	"msg-prot.h"
+#include	"msg-prot.h"
 
 #include "PcgDun.h"
 #include "FileList.h"
@@ -121,6 +121,9 @@ extern WSCvscrBar *MapScrollY;
 // ȥΥǥ쥯ȥ
 #define STR_TITLE_DIR_NAME	"title/"
 
+#define TITLE_FONT_POINT	32
+#define TITLE_FONT_DOT	32
+
 ////////////////////////////////////////////////////////////////
 // ѥ󡦥ǥåΥơ֥
 ////////////////////////////////////////////////////////////////
@@ -179,6 +182,22 @@ pos_t gIdxWallDiagonal[2 * 2 * 2 * 2] = {
 	{ +1, +1 }, //SW SE NW NE
 };
 
+////////////////////////////////////////////////////////////////
+
+static const char *gStrTitle[3] = {
+	"Labyrinths",
+	"   and    ",
+	" Legends  ",
+};
+
+typedef enum {
+	DRAW_TITLE_KIND_0,
+	DRAW_TITLE_KIND_1,
+	DRAW_TITLE_KIND_MAX_N,
+} drawTitleKindType;
+
+////////////////////////////////////////////////////////////////
+
 #ifdef D_GTK
 static GdkFont *gSelFont;
 
@@ -213,6 +232,8 @@ PcgDun::PcgDun()
 	nIntervalWidth = 1;
 	nIntervalHeight = 1;
 
+	pTitleStyle = NULL;
+
 	bFlgEnaDrawTurn = false;
 
 #ifdef D_MFC
@@ -240,6 +261,11 @@ PcgDun::~PcgDun()
 		pSignboardFont = NULL;
 	}
 
+	if( pTitleFont != NULL ){
+		delete pTitleFont;
+		pTitleFont = NULL;
+	}
+
 	if( pDc != NULL ){
 		delete pDc;
 		pDc = NULL;
@@ -300,6 +326,65 @@ void PcgDun::initTitle()
 			}
 		}
 	}
+
+#ifdef D_GTK
+	// ǥեȤΥ
+
+	if( pTitleStyle == NULL )
+		pTitleStyle = gtk_style_copy( gtk_widget_get_default_style() );
+	if( pTitleStyle->fg_gc[GTK_STATE_NORMAL] == NULL ){
+		pTitleStyle->fg_gc[GTK_STATE_NORMAL]
+				= gdk_gc_new( gMapDrawingArea->window );
+	}
+	if( pTitleStyle->bg_gc[GTK_STATE_NORMAL] == NULL ){
+		pTitleStyle->bg_gc[GTK_STATE_NORMAL]
+				= gdk_gc_new( gMapDrawingArea->window );
+	}
+
+	// եȤ
+
+	WSCstring sFontPoint = TITLE_FONT_POINT * 10;
+	WSCstring sFontName = "";
+	sFontName += "-*-*-*-i-normal--*-";
+	sFontName += sFontPoint;
+	sFontName += "-*,-*";
+
+	gtk_style_set_font( pTitleStyle, gdk_fontset_load( sFontName ) );
+#endif // D_GTK
+
+#ifdef D_MFC
+	// ǥեȤΥ
+
+	if( pTitleStyle == NULL )
+		pTitleStyle = (void *)1;
+
+	// եȤ
+
+	LONG h = TITLE_FONT_DOT;
+	mTitleFontDesc.lfHeight = h;
+	mTitleFontDesc.lfWidth = 0;
+	mTitleFontDesc.lfEscapement = 0;
+	mTitleFontDesc.lfOrientation = 0;
+	mTitleFontDesc.lfWeight = FW_NORMAL;
+	mTitleFontDesc.lfItalic = FALSE;
+	mTitleFontDesc.lfUnderline = FALSE;
+	mTitleFontDesc.lfStrikeOut = FALSE;
+	mTitleFontDesc.lfCharSet = SHIFTJIS_CHARSET;
+	mTitleFontDesc.lfOutPrecision
+			= OUT_DEFAULT_PRECIS;
+	mTitleFontDesc.lfClipPrecision
+			= CLIP_DEFAULT_PRECIS;
+	mTitleFontDesc.lfQuality = DEFAULT_QUALITY;
+	mTitleFontDesc.lfPitchAndFamily
+			= (FIXED_PITCH | FF_MODERN);
+	strcpy( mTitleFontDesc.lfFaceName, "Pica" );
+
+	pTitleFont = new CFont;
+	pTitleFont->CreateFontIndirect(
+			&mTitleFontDesc );
+
+	mD3Title.Create( &mTitleFontDesc );
+#endif // D_MFC
 }
 
 ////////////////////////////////////////////////////////////////
@@ -323,7 +408,10 @@ void PcgDun::init( GraphConf *cnf )
 
 	// ѥ󸡺
 
-	WSCstring dir = STR_DEFAULT_GRAPH_DIR_NAME;
+	WSCstring path = cnf->getDir();
+	long w = path.getWords( "/" );
+	WSCstring dir = path.getWord( w - 1, "/" );
+
 	WSCstring ext = STR_GRAPH_FILE_EXT;
 
 	FileList::setStrDirSelGraph( dir );
@@ -836,7 +924,7 @@ void PcgDun::initGraphConf( GraphConf *cnf )
 	else
 		bFlgEnaDrawTurn = true;
 #else // D_MFC
-	bFlgEnaDrawTurn = false;
+	bFlgEnaDrawTurn = true;
 #endif // D_MFC
 }
 
@@ -1221,6 +1309,8 @@ bool PcgDun::drawScroll( long x, long y, long w, long h )
 	if( chk_nest_flg_dun() )
 		return false;
 
+	sort_all_chr();
+
 	long sizX = getTileSizeX( true );
 	long sizY = getTileSizeY( true );
 
@@ -1241,7 +1331,16 @@ bool PcgDun::drawScroll( long x, long y, long w, long h )
 		}
 	}
 
+#ifdef D_MFC
+	if( g_Dir3d.GetFlagRefresh() ){
+		flush( mapX1, mapY1, mapW, mapH );
+	}
+
+	setUpdateFlgRange( mapX1, mapY1, mapW, mapH, true );
+	g_Dir3d.SetFlagRefresh( +1 );
+#else // D_MFC
 	flush( mapX1, mapY1, mapW, mapH );
+#endif // D_MFC
 
 	return true;
 }
@@ -1571,6 +1670,7 @@ bool PcgDun::clipWinY( long mapY )
 
 ////////////////////////////////////////////////////////////////
 // ޥåפκ
+// bool flgForce : 
 ////////////////////////////////////////////////////////////////
 
 void PcgDun::redraw( bool flgForce )
@@ -1578,7 +1678,8 @@ void PcgDun::redraw( bool flgForce )
 	if( bFlgEnaDrawTurn && !flgForce )
 		return;
 
-	draw( 0, 0, MAP_MAX_X, MAP_MAX_Y, flgForce );
+	setUpdateFlgRange( 0, 0, MAP_MAX_X, MAP_MAX_Y, true );
+	drawTurn( true );
 }
 
 ////////////////////////////////////////////////////////////////
@@ -1587,6 +1688,7 @@ void PcgDun::redraw( bool flgForce )
 // long mapY : Y ɸ
 // long mapW : 
 // long mapH : ⤵
+// bool flgForce : 
 // return : 顼̵ä
 ////////////////////////////////////////////////////////////////
 
@@ -1603,17 +1705,29 @@ bool PcgDun::draw(
 	if( chk_nest_flg_dun() )
 		return false;
 
+#ifdef D_MFC
+	if( g_Dir3d.GetFlagRefresh() ){
+		drawSub( mapX, mapY, mapW, mapH );
+		flush( mapX, mapY, mapW, mapH );
+	}
+#else // D_MFC
 	drawSub( mapX, mapY, mapW, mapH );
 	flush( mapX, mapY, mapW, mapH );
+#endif // D_MFC
+
+#ifdef D_MFC
+	g_Dir3d.SetFlagRefresh( +1 );
+#endif // D_MFC
 
 	return true;
 }
 
 ////////////////////////////////////////////////////////////////
 // Υޥåפ
+// bool flgForce : 
 ////////////////////////////////////////////////////////////////
 
-void PcgDun::drawTurn()
+void PcgDun::drawTurn( bool flgForce )
 {
 	long x = getScrollBarX();
 	long y = getScrollBarY();
@@ -1630,9 +1744,7 @@ void PcgDun::drawTurn()
 	long mapW = mapX2 - mapX1;
 	long mapH = mapY2 - mapY1;
 
-	setUpdateFlgRange( mapX1, mapY1, mapW, mapH, true );
-
-	if( !bFlgEnaDrawTurn )
+	if( !bFlgEnaDrawTurn && !flgForce )
 		return;
 
 	if( chk_nest_flg_dun() ){
@@ -1641,8 +1753,19 @@ void PcgDun::drawTurn()
 		return;
 	}
 
+#ifdef D_MFC
+	if( g_Dir3d.GetFlagRefresh() ){
+		drawSub( mapX1, mapY1, mapW, mapH );
+		flush( mapX1, mapY1, mapW, mapH );
+	}
+#else // D_MFC
 	drawSub( mapX1, mapY1, mapW, mapH );
 	flush( mapX1, mapY1, mapW, mapH );
+#endif // D_MFC
+
+#ifdef D_MFC
+	g_Dir3d.SetFlagRefresh( +1 );
+#endif // D_MFC
 }
 
 ////////////////////////////////////////////////////////////////
@@ -1678,9 +1801,7 @@ bool PcgDun::drawSub( long mapX, long mapY, long mapW, long mapH )
 
 	for( long y = mapY1; y < mapY2; y++ ){
 		for( long x = mapX1; x < mapX2; x++ ){
-			if( clipWin( x, y ) ){
-				setUpdateFlg( x, y, false );
-			} else {
+			if( !clipWin( x, y ) ){
 				setUpdateFlg( x, y, true );
 			}
 		}
@@ -1690,11 +1811,20 @@ bool PcgDun::drawSub( long mapX, long mapY, long mapW, long mapH )
 
 	for( long y = objY1; y < objY2; y++ ){
 		for( long x = objX1; x < objX2; x++ ){
+			if( !getUpdateFlg( x, y ) )
+				continue;
 			if( !clipWin( x, y ) )
 				continue;
 
-			if( !drawObj( x, y ) )
+#ifdef D_MFC
+			drawObj( x, y );
+			setUpdateFlg( x, y, true );
+#else // D_MFC
+			if( drawObj( x, y ) )
+				setUpdateFlg( x, y, false );
+			else
 				setUpdateFlg( x, y, true );
+#endif // D_MFC
 		}
 	}
 
@@ -1750,7 +1880,7 @@ bool PcgDun::drawSub( long mapX, long mapY, long mapW, long mapH )
 
 	// 饯
 
-	drawChrListAll( sx, sy, sw,sh );
+	drawChrListAll( sx, sy, sw, sh );
 
 	// 
 
@@ -1938,6 +2068,8 @@ void PcgDun::initText()
 #endif // D_GTK
 
 #ifdef D_MFC
+	// ǥեȤΥ
+
 	if( pStyle == NULL )
 		pStyle = (void *)1;
 
@@ -1946,10 +2078,6 @@ void PcgDun::initText()
 
 	CXlnlView *view = theApp.m_pMainFrm->GetView();
 
-	// ǥեȤΥ
-
-	// եȤ
-
 	// եȡ
 
 #if FLAG_FONT_TO_TEXT_SIZE
@@ -1965,6 +2093,32 @@ void PcgDun::initText()
 	nTextSizeY = nTextSizeY * _100_PERCENT / nTileSizeRate;
 #endif
 
+	// եȤ
+
+	LONG h = g_text_mode_point;
+	h = h * g_Dir3d.nTileSizeRate / _100_PERCENT;
+	mTextFontDesc.lfHeight = h;
+	mTextFontDesc.lfWidth = 0;
+	mTextFontDesc.lfEscapement = 0;
+	mTextFontDesc.lfOrientation = 0;
+	mTextFontDesc.lfWeight = FW_NORMAL;
+	mTextFontDesc.lfItalic = FALSE;
+	mTextFontDesc.lfUnderline = FALSE;
+	mTextFontDesc.lfStrikeOut = FALSE;
+	mTextFontDesc.lfCharSet = SHIFTJIS_CHARSET;
+	mTextFontDesc.lfOutPrecision
+			= OUT_DEFAULT_PRECIS;
+	mTextFontDesc.lfClipPrecision
+			= CLIP_DEFAULT_PRECIS;
+	mTextFontDesc.lfQuality = DEFAULT_QUALITY;
+	mTextFontDesc.lfPitchAndFamily
+			= (FIXED_PITCH | FF_MODERN);
+	strcpy( mTextFontDesc.lfFaceName, "Pica" );
+
+	pTextFont = new CFont;
+	pTextFont->CreateFontIndirect(
+			&mTextFontDesc );
+
 	// 롦СĴ
 
 	view->ResetScroll();
@@ -2181,6 +2335,8 @@ bool PcgDun::drawText( long mapX, long mapY, long mapW, long mapH )
 
 			CPen *pen = new CPen();
 			SetPenAttrFg( pDc, pen, NULL, &attr );
+			CFont *pPrevFont = pDc->SelectObject(
+					pTextFont );
 
 			s[0] = dun->map.total.mjr[my][mx];
 			pDc->TextOut( mjrX, y, s );
@@ -2188,6 +2344,7 @@ bool PcgDun::drawText( long mapX, long mapY, long mapW, long mapH )
 			s[0] = dun->map.total.mnr[my][mx];
 			pDc->TextOut( mnrX, y, s );
 
+			pDc->SelectObject( pPrevFont );
 			delete pen;
 
 			// 轪λ
@@ -2357,14 +2514,271 @@ bool PcgDun::drawTitle()
 	if( nTitleRandmSelN <= -1 )
 		return false;
 
+	// طʤɤĤ֤
+
+#ifdef D_GTK
+	GdkDrawable *d = getWBuf()->getPixMap();
+	GdkGC *gc = pTitleStyle->bg_gc[GTK_STATE_NORMAL];
+	gdk_draw_rectangle( d, gc, TRUE,
+			0, 0,
+			gMapDrawingArea->allocation.width,
+			gMapDrawingArea->allocation.height );
+#endif // D_GTK
+
+#ifdef D_MFC
+#endif // D_MFC
+
+	// طʲ
+
 	long x = gPcgDun.getScrollBarX();
 	long y = gPcgDun.getScrollBarY();
 	long w = pcgTitle.getWidth();
 	long h = pcgTitle.getHeight();
+	pcgTitle.drawFix( getWBuf(), x, y, w, h );
 
-	return pcgTitle.drawFix( getWBuf(), x, y, w, h );
+	// ȥ롦͡
+
+	static time_t t = 0;
+	static long frame = 0;
+	static drawTitleKindType kind = DRAW_TITLE_KIND_1;
+	if( time( NULL ) >= t ){
+		frame = 0;
+		kind = (drawTitleKindType)randm( DRAW_TITLE_KIND_MAX_N );
+	}
+	if( frame <= 0 )
+		t = time( NULL ) + 10;
+
+	switch( kind ){
+	case DRAW_TITLE_KIND_0:
+		drawTitle0( frame );
+		break;
+	case DRAW_TITLE_KIND_1:
+		drawTitle1( frame );
+		break;
+	case DRAW_TITLE_KIND_MAX_N:
+		break;
+	}
+#ifdef D_MFC
+	g_Dir3d.SetFlagRefresh( +1 );
+#endif // D_MFC
+
+	frame++;
+
+	return true;
 }
 
+void PcgDun::drawTitle0( long frame )
+{
+	if( pTitleStyle == NULL )
+		initTitle();
+
+	long len = 1;
+	for( long j = 0; j < 3; j++ )
+		len = max_l( len, str_len_draw( gStrTitle[j] ) );
+
+#ifdef D_GTK
+	const long bgnX = gMapDrawingArea->allocation.width;
+	const long baseX = (gMapDrawingArea->allocation.width
+			- (len * TITLE_FONT_DOT)) / 2;
+	const long baseY = gMapDrawingArea->allocation.height / 2;
+#endif // D_GTK
+
+#ifdef D_MFC
+	CXlnlView *view = theApp.m_pMainFrm->GetView();
+	CRect rect;
+	view->GetClientRect( &rect );
+
+	const long bgnX = rect.Width();
+	const long baseX = (rect.Width() / 2)
+			- (len * TITLE_FONT_DOT / 2);
+	const long baseY = rect.Height() / 2;
+#endif // D_MFC
+
+	const long v = 64;
+
+	long x[3], y[3];
+	long idx[3];
+	x[0] = bgnX;
+	x[1] = bgnX;
+	x[2] = bgnX;
+	y[0] = baseY - (TITLE_FONT_DOT * 120 / _100_PERCENT);
+	y[1] = baseY;
+	y[2] = baseY + (TITLE_FONT_DOT * 120 / _100_PERCENT);
+	for( long j = 0; j < 3; j++ )
+		idx[j] = 0;
+
+	for( long f = 0; f < frame; f++ ){
+		if( (gStrTitle[0][idx[0]] == '\0')
+				&& (gStrTitle[1][idx[1]] == '\0')
+				&& (gStrTitle[2][idx[2]] == '\0') ){
+			break;
+		}
+
+		for( long j = 0; j < 3; j++ ){
+			x[j] -= v;
+			if( x[j] < (baseX + (idx[j] * TITLE_FONT_DOT)) ){
+				x[j] = (baseX + (idx[j] * TITLE_FONT_DOT));
+				char c;
+				c = gStrTitle[j][idx[j]];
+				if( c != '\0' ){
+					drawTitleCharStd( x[j], y[j], c );
+					idx[j]++;
+					x[j] = bgnX;
+				}
+			}
+		}
+	}
+
+	for( long j = 0; j < 3; j++ ){
+		if( gStrTitle[j][idx[j]] != '\0' ){
+			drawTitleCharStd( x[j], y[j],
+					gStrTitle[j][idx[j]] );
+		}
+	}
+}
+
+void PcgDun::drawTitle1( long frame )
+{
+	if( pTitleStyle == NULL )
+		initTitle();
+
+	long len = 1;
+	for( long i = 0; i < 3; i++ )
+		len = max_l( len, str_len_draw( gStrTitle[i] ) );
+
+#ifdef D_GTK
+	const long w = gMapDrawingArea->allocation.width;
+	const long h = gMapDrawingArea->allocation.height;
+#endif // D_GTK
+
+#ifdef D_MFC
+	CXlnlView *view = theApp.m_pMainFrm->GetView();
+	CRect rect;
+	view->GetClientRect( &rect );
+
+	const long w = rect.Width();
+	const long h = rect.Height();
+#endif // D_MFC
+
+	const long baseX = (w - (len * TITLE_FONT_DOT)) / 2;
+	const long baseY = h / 2;
+
+	long mx[3][16], my[3][16];
+	long x[3][16], y[3][16];
+	for( long j = 0; j < len; j++ ){
+		mx[0][j] = (baseX + (j * TITLE_FONT_DOT));
+		mx[1][j] = (baseX + (j * TITLE_FONT_DOT));
+		mx[2][j] = (baseX + (j * TITLE_FONT_DOT));
+		my[0][j] = baseY - (TITLE_FONT_DOT * 120 / _100_PERCENT);
+		my[1][j] = baseY;
+		my[2][j] = baseY + (TITLE_FONT_DOT * 120 / _100_PERCENT);
+
+		long xx = (w / 2)
+				+ ((len / 2 - j) * TITLE_FONT_DOT);
+		x[0][j] = xx;
+		x[1][j] = xx;
+		x[2][j] = xx;
+		y[0][j] = 0 - TITLE_FONT_DOT;
+		y[1][j] = h / 2;
+		y[2][j] = h + TITLE_FONT_DOT;
+	}
+
+	for( long f = 0; f < frame; f++ ){
+		for( long i = 0; i < 3; i++ ){
+			for( long j = 0; j < len; j++ ){
+				x[i][j] += (mx[i][j] - x[i][j]) / 16;
+				y[i][j] += (my[i][j] - y[i][j]) / 16;
+				x[i][j] += sgn_l( mx[i][j] - x[i][j] );
+				y[i][j] += sgn_l( my[i][j] - y[i][j] );
+			}
+		}
+	}
+
+	for( long i = 0; i < 3; i++ ){
+		for( long j = 0; j < len; j++ ){
+			if( gStrTitle[i][j] == '\0' )
+				break;
+
+			drawTitleCharStd( x[i][j], y[i][j],
+					gStrTitle[i][j] );
+		}
+	}
+}
+
+void PcgDun::drawTitleCharStd( long x, long y, char c )
+{
+#ifdef D_GTK
+	GdkColor color;
+
+	// 
+
+	color.red = 0x4444;
+	color.green = 0x4444;
+	color.blue = 0x4444;
+	drawTitleChar( color, x + 2, y + 2, c );
+
+	color.red = 0xaaaa;
+	color.green = 0x8888;
+	color.blue = 0x8888;
+	drawTitleChar( color, x - 2, y - 2, c );
+
+	// 
+
+	color.red = 0x8888;
+	color.green = 0x0000;
+	color.blue = 0x0000;
+	drawTitleChar( color, x, y, c );
+#endif // D_GTK
+
+#ifdef D_MFC
+	D3DCOLOR color;
+
+	// 
+
+	color = 0xff444444;
+	drawTitleChar( color, x + 2, y + 2, c );
+
+	color = 0xffaa8888;
+	drawTitleChar( color, x - 2, y - 2, c );
+
+	// 
+
+	color = 0xff880000;
+	drawTitleChar( color, x, y, c );
+#endif // D_MFC
+}
+
+#ifdef D_GTK
+void PcgDun::drawTitleChar( GdkColor color, long x, long y, char c )
+{
+	char str[4 + 1];
+	str[0] = c;
+	str[1] = '\0';
+
+	GdkDrawable *d = getWBuf()->getPixMap();
+	GdkFont *font = gtk_style_get_font( pTitleStyle );
+	GdkGC *gc = pTitleStyle->fg_gc[GTK_STATE_NORMAL];
+
+	gdk_color_alloc( gdk_colormap_get_system(), &color );
+	gdk_gc_set_foreground( gc, &color );
+
+	gdk_draw_string( d, font, gc, x, y, str );
+}
+#endif // D_GTK
+
+#ifdef D_MFC
+void PcgDun::drawTitleChar( D3DCOLOR color, long x, long y, char c )
+{
+	char str[4 + 1];
+	str[0] = c;
+	str[1] = '\0';
+
+	mD3Title.Begin();
+	mD3Title.Draw( x, y, color, str );
+	mD3Title.End();
+}
+#endif // D_MFC
+
 ////////////////////////////////////////////////////////////////
 // 饯
 // long mapX : X ɸ
