@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2007 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: PcgDun.h,v 1.33 2007/12/24 16:37:22 bowkenken Exp $
+* $Id: PcgDun.h,v 1.38 2008/02/12 23:17:25 bowkenken Exp $
 ***************************************************************/
 
 #ifndef PCG_DUN_H
@@ -285,9 +285,27 @@ private:
 #ifdef D_MFC
 	// եȤΥ
 	void *pStyle;
+	LOGFONT mTextFontDesc;
+	CFont *pTextFont;
 	CClientDC *pDc;
 #endif // D_MFC
 
+	//// ȥ ////
+
+#ifdef D_GTK
+	// եȤΥ
+	GtkStyle *pTitleStyle;
+#endif // D_GTK
+
+#ifdef D_MFC
+	// եȤΥ
+	void *pTitleStyle;
+	LOGFONT mTitleFontDesc;
+	CFont *pTitleFont;
+
+	CDir3dText mD3Title;
+#endif // D_MFC
+
 	// ˥ޥåפ褹뤫?
 	bool bFlgEnaDrawTurn;
 
@@ -330,7 +348,7 @@ public:
 	void redraw( bool flgForce = false );
 	bool draw( long mapX, long mapY, long mapW, long mapH,
 		bool flgForce = false );
-	void drawTurn();
+	void drawTurn( bool flgForce = false );
 	bool drawSub( long mapX, long mapY, long mapW, long mapH );
 	void flush( long mapX, long mapY, long mapW, long mapH );
 
@@ -356,6 +374,15 @@ public:
 	long getObjDrawHeight( long mapY, Pcg *pPcg );
 
 	bool drawTitle();
+	void drawTitle0( long frame );
+	void drawTitle1( long frame );
+	void drawTitleCharStd( long x, long y, char c );
+#ifdef D_GTK
+	void drawTitleChar( GdkColor color, long x, long y, char c );
+#endif // D_GTK
+#ifdef D_MFC
+	void drawTitleChar( COLORREF color, long x, long y, char c );
+#endif // D_MFC
 
 private:
 #ifdef D_MFC
