@@ -33,6 +33,8 @@ bool_t	load_game_conf_id_brace(
 );
 bool_t	load_game_conf_lang( char *id, char *str );
 bool_t	load_game_conf_clk_tck( char *id, long val );
+bool_t	load_game_conf_screen_size_auto( char *id, long val );
+bool_t	load_game_conf_screen_size_lines( char *id, long val );
 bool_t	load_game_conf_scroll( char *id, char *str );
 bool_t	load_game_conf_clr_key_buf_every( char *id, long val );
 bool_t	load_game_conf_clr_key_buf_menu( char *id, long val );
@@ -65,6 +67,7 @@ bool_t	save_game_conf_ver_1( const ver_t *ver );
 bool_t	save_game_conf_contents( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_language( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_frame_rate( FILE *fp, const ver_t *ver );
+bool_t	save_game_conf_screen_size( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_scroll( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_flg( FILE *fp, const ver_t *ver );
 bool_t	save_game_conf_vfx( FILE *fp, const ver_t *ver );
