@@ -113,6 +113,9 @@ void	set_scroll_smooth( void );
 void	set_scroll_center( void );
 char	*cb_menu_redraw_every_turn( menu_t **pp, menu_t **dflt );
 char	*cb_menu_mul_msg_line( menu_t **pp, menu_t **dflt );
+char	*cb_menu_screen_size_init( menu_t **pp, menu_t **dflt );
+char	*cb_menu_screen_size_auto( menu_t **pp, menu_t **dflt );
+char	*cb_menu_screen_size_lines( menu_t **pp, menu_t **dflt );
 char	*cb_menu_vfx( menu_t **pp, menu_t **dflt );
 char	*cb_menu_vfx_mbr( menu_t **pp, menu_t **dflt );
 char	*cb_menu_vfx_mnstr( menu_t **pp, menu_t **dflt );
