@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2007 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: item.h,v 1.88 2007/02/28 21:40:49 bowkenken Exp $
+* $Id: item.h,v 1.91 2008/02/14 08:44:16 bowkenken Exp $
 ***************************************************************/
 
 #ifndef	ITEM_H
@@ -258,7 +258,9 @@ typedef enum {
 	EQUIP_KIND_LEFT_HAND,
 	EQUIP_KIND_FEET,
 	EQUIP_KIND_LIGHT_SOURCE,
+
 	EQUIP_KIND_MAX_N,
+	EQUIP_KIND_NULL = EQUIP_KIND_WIELDING,
 } equip_kind_t;
 
 /* μ */
@@ -541,6 +543,8 @@ typedef enum {
 	SCROLL_KIND_INUNDATION,
 	SCROLL_KIND_SUBSTITUTE,
 	SCROLL_KIND_BURNING_HANDS,
+	SCROLL_KIND_RALLY,
+	SCROLL_KIND_IRON_MAN,
 	SCROLL_KIND_MIND_READING,
 	SCROLL_KIND_MAX_N,
 } scroll_kind_t;
