@@ -24,7 +24,7 @@ void	handle_map_scroll_changed(
 	GtkAdjustment *adjustment,
 	gpointer user_data
 );
-gint	handle_map_redraw( gpointer p );
+gint	handle_map_draw( gpointer p );
 gint	handle_map_motion_notify(
 	GtkWidget *widget, GdkEventMotion *event
 );
