@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2007 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: main-gtk.cpp,v 1.32 2007/12/16 02:09:34 bowkenken Exp $
+* $Id: main-gtk.cpp,v 1.34 2008/02/12 20:23:25 bowkenken Exp $
 ***************************************************************/
 
 ////////////////////////////////////////////////////////////////
@@ -52,6 +52,8 @@
 
 GtkWidget *btnMapExec;
 
+guint ghTimer;
+
 ////////////////////////////////////////////////////////////////
 // ᥤ󡦥롼
 // int argc : ο
@@ -122,15 +124,9 @@ void	init_gtk_win( int argc, char **argv )
 
 	// ե졼ࡦ졼Ȥ
 
-#if	0
-	gtk_timeout_add( 1000 * 1 / 60, handle_map_redraw,
-			gMapDrawingArea );
-#elif	0
-	gtk_timeout_add( 1000 * 20 / 60, handle_map_redraw,
-			gMapDrawingArea );
-#else
-//@@@
-#endif
+	ghTimer = gtk_timeout_add( 1000 * 2 / 60,
+			handle_map_draw,
+			(gpointer)gMapDrawingArea );
 
 	// ȥ
 
@@ -154,6 +150,7 @@ void	init_main_sel_conf( GraphConf *cnf )
 
 	gPcgDun.init( cnf );
 
+	gtk_timeout_remove( ghTimer );
 	change_scene( SCENE_N_TOWN );
 
 	gGuiMenu.init();
@@ -532,8 +529,10 @@ void	handle_map_scroll_changed(
 // return : ?
 ////////////////////////////////////////////////////////////////
 
-gint	handle_map_redraw( gpointer p )
+gint	handle_map_draw( gpointer p )
 {
+#if 0
+//@@@
 	GtkWidget *drawingArea = (GtkWidget *)p;
 	GdkGC *gc = drawingArea->style->fg_gc[GTK_STATE_NORMAL];
 
@@ -554,6 +553,18 @@ gint	handle_map_redraw( gpointer p )
 			x, y,
 			0, 0,
 			w, h );
+#else
+	if( chk_scene_group( SCENE_GROUP_N_TITLE ) )
+		gPcgDun.drawTitle();
+
+	GtkWidget *drawingArea = (GtkWidget *)p;
+	GdkRectangle rect;
+	rect.x = 0;
+	rect.y = 0;
+	rect.width = drawingArea->allocation.width;
+	rect.height = drawingArea->allocation.height;
+	gtk_widget_draw( GTK_WIDGET( drawingArea ), &rect );
+#endif
 
 	return TRUE;
 }
