@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2007 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: menu.c,v 1.191 2007/02/28 22:33:24 bowkenken Exp $
+* $Id: menu.c,v 1.202 2008/02/14 23:41:53 bowkenken Exp $
 ***************************************************************/
 
 #define	MENU_C
@@ -35,11 +35,12 @@
 
 #define	DRAW_MENU_CLR_N	-1
 
-#define	FUNC_KEY_MAX_N	15
-
 /**/
 
+static long	g_nest_flg_executing_menu;
 static bool_t	g_flg_space_select;
+static bool_t	g_flg_menu_dflt;
+static bool_t	g_flg_chk_menu;
 static bool_t	g_flg_chk_short_key;
 
 static ask_t	g_sel_ask;
@@ -74,16 +75,26 @@ static char	*menu_kind_ls[] = {
 
 menu_t	dummy_menu_close_menu;
 
-static char	g_func[FUNC_KEY_MAX_N + 1][MACRO_MAX_LEN + 1];
+/***************************************************************
+* ե󥯥󡦥
+***************************************************************/
 
+#define	MENU_FUNC_MAX_N_PER_PAGE	10
+
+static char	g_func[FUNC_KEY_MAX + 1][MACRO_MAX_LEN + 1];
+
+/* ǥեȤΥե󥯥󡦥 */
 typedef struct {
 	char	*key;
 	char	*dflt;
 } func_short_key_t;
 
+/* ǥեȤΥե󥯥󡦥Υơ֥ */
+/* Խ func_n_t ⽤ */
 static func_short_key_t
-		g_ls_func_short_key[FUNC_KEY_MAX_N + 1] = {
+		g_ls_func_short_key[FUNC_KEY_MAX + 1] = {
 	{ NULL, NULL },	/* NULL */
+
 	{ NULL, NULL },	/* F1 */
 	{ NULL, NULL },	/* F2 */
 	{ NULL, NULL },	/* F3 */
@@ -94,8 +105,49 @@ static func_short_key_t
 	{ NULL, "v" },	/* F8 */
 	{ NULL, NULL },	/* F9 */
 	{ NULL, NULL },	/* F10 */
-	{ NULL, "," },	/* F11 */
-	{ NULL, "," },	/* F12 */
+	{ NULL, NULL },	/* F11 */
+	{ NULL, NULL },	/* F12 */
+	{ NULL, NULL },	/* F13 */
+	{ NULL, NULL },	/* F14 */
+	{ NULL, NULL },	/* F15 */
+	{ NULL, NULL },	/* F16 */
+	{ NULL, NULL },	/* F17 */
+	{ NULL, NULL },	/* F18 */
+	{ NULL, NULL },	/* F19 */
+	{ NULL, NULL },	/* F20 */
+
+	{ NULL, "\n" },	/* F21 */
+	{ NULL, "+" },	/* F22 */
+	{ NULL, "." },	/* F23 */
+	{ NULL, NULL },	/* F24 */
+	{ NULL, NULL },	/* F25 */
+	{ NULL, NULL },	/* F26 */
+	{ NULL, NULL },	/* F27 */
+	{ NULL, NULL },	/* F28 */
+	{ NULL, NULL },	/* F29 */
+	{ NULL, NULL },	/* F30 */
+
+	{ NULL, "+" },	/* F31 */
+	{ NULL, "," },	/* F32 */
+	{ NULL, NULL },	/* F33 */
+	{ NULL, NULL },	/* F34 */
+	{ NULL, NULL },	/* F35 */
+	{ NULL, NULL },	/* F36 */
+	{ NULL, NULL },	/* F37 */
+	{ NULL, NULL },	/* F38 */
+	{ NULL, NULL },	/* F39 */
+	{ NULL, NULL },	/* F40 */
+
+	{ NULL, "/" },	/* F41 */
+	{ NULL, "*" },	/* F42 */
+	{ NULL, "-" },	/* F43 */
+	{ NULL, "+" },	/* F44 */
+	{ NULL, NULL },	/* F45 */
+	{ NULL, NULL },	/* F46 */
+	{ NULL, NULL },	/* F47 */
+	{ NULL, NULL },	/* F48 */
+	{ NULL, NULL },	/* F49 */
+	{ NULL, NULL },	/* F50 */
 };
 
 static bool_t	g_flg_dup_short_key[256];
@@ -104,11 +156,14 @@ static bool_t	g_flg_dup_short_key[256];
 
 #include	"menu-tab.h"
 
-/**/
+/***************************************************************/
 
 void	init_var_menu( void )
 {
+	g_nest_flg_executing_menu = make_nest_flg();
+
 	set_flg_space_select( FALSE );
+	set_flg_menu_dflt( TRUE );
 }
 
 /**/
@@ -117,11 +172,12 @@ void	init_menu( void )
 {
 	long	i;
 
+	g_flg_chk_menu = TRUE;
 	g_flg_chk_short_key = FALSE;
 
 	chk_dup_short_key();
 
-	for( i = 1; i <= FUNC_KEY_MAX_N; i++ ){
+	for( i = 0; i <= FUNC_KEY_MAX; i++ ){
 		g_func[i][0] = '\0';
 
 		if( g_ls_func_short_key[i].dflt != NULL ){
@@ -139,6 +195,7 @@ void	init_menu( void )
 void	init_ls_menu_func( void )
 {
 	menu_t	*ls;
+	char	key;
 	long	i, k;
 
 	ls = ls_menu_func;
@@ -164,69 +221,112 @@ void	init_ls_menu_func( void )
 	ls[k].flg = FLG_NULL;
 	k++;
 
-	for( i = 1; i <= FUNC_KEY_MAX_N; i++ ){
-		sprintf( ls[k].path, "/func/%ld", i );
+	key = 'A';
+	for( i = 0; i < FUNC_KEY_MAX; i++ ){
+		long	fn;
+
+		fn = i + 1;
+
+		if( (i != 0) && ((i % MENU_FUNC_MAX_N_PER_PAGE) == 0) ){
+			str_nz_cpy( ls[k].path, "/func/page separator",
+					MENU_PATH_MAX_LEN );
+			str_nz_cpy( ls[k].kind, "<PageSeparator>",
+				MENU_KIND_MAX_LEN );
+			ls[k].accel_key = '\0';
+			ls[k].short_key[0] = '\0';
+			ls[k].n_msg = N_MSG_NULL;
+			ls[k].msg[0] = '\0';
+			ls[k].call_back = NULL;
+			ls[k].arg = NULL;
+			ls[k].flg = FLG_NULL;
+			k++;
+			key = 'A';
+		}
+
+		sprintf( ls[k].path, "/func/%ld", fn );
 		str_nz_cpy( ls[k].kind, "<Branch>", MENU_KIND_MAX_LEN );
-		ls[k].accel_key = (char)('A' + (i - 1));
+		ls[k].accel_key = key;
 		ls[k].short_key[0] = '\0';
-		sprintf( ls[k].msg, MSG_MENU_FUNC_KEY, i );
+		sprintf( ls[k].msg, MSG_MENU_FUNC_KEY, fn );
 		ls[k].call_back = NULL;
 		ls[k].arg = NULL;
 		ls[k].flg = FLG_NULL;
 		k++;
 
-		sprintf( ls[k].path, "/func/%ld/--", i );
+		sprintf( ls[k].path, "/func/%ld/--", fn );
 		str_nz_cpy( ls[k].kind, "<Title>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = '\0';
 		ls[k].short_key[0] = '\0';
-		sprintf( ls[k].msg, MSG_MENU_FUNC_KEY, i );
+		sprintf( ls[k].msg, MSG_MENU_FUNC_KEY, fn );
 		ls[k].call_back = NULL;
 		ls[k].arg = NULL;
 		ls[k].flg = FLG_NULL;
 		k++;
 
-		sprintf( ls[k].path, "/func/%ld/exec", i );
+		sprintf( ls[k].path, "/func/%ld/exec", fn );
 		str_nz_cpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = 'X';
-		if( g_ls_func_short_key[i].key == NULL ){
-			sprintf( ls[k].short_key, "&%02ld", i );
+		if( g_ls_func_short_key[fn].key == NULL ){
+			sprintf( ls[k].short_key, "&%02ld", fn );
 		} else {
 			sprintf( ls[k].short_key, "%s &%02ld",
-					g_ls_func_short_key[i].key, i );
+					g_ls_func_short_key[i].key, fn );
 		}
 		str_nz_cpy( ls[k].msg, MSG_MENU_FUNC_EXEC, MENU_MSG_MAX_LEN );
 		ls[k].call_back = cb_menu_func_exec;
-		ls[k].arg = (void *)i;
+		ls[k].arg = (void *)fn;
 		ls[k].flg = FLG_NULL;
 		k++;
 
-		sprintf( ls[k].path, "/func/%ld/edit", i );
+		sprintf( ls[k].path, "/func/%ld/edit", fn );
 		str_nz_cpy( ls[k].kind, "<OpenWinItem>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = 'E';
 		ls[k].short_key[0] = '\0';
 		str_nz_cpy( ls[k].msg, MSG_MENU_FUNC_EDIT, MENU_MSG_MAX_LEN );
 		ls[k].call_back = cb_menu_func_edit;
-		ls[k].arg = (void *)i;
+		ls[k].arg = (void *)fn;
 		ls[k].flg = FLG_NULL;
 		k++;
 
-		sprintf( ls[k].path, "/func/%ld/set reg macro", i );
+		sprintf( ls[k].path, "/func/%ld/set reg macro", fn );
 		str_nz_cpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
 		ls[k].accel_key = 'M';
 		ls[k].short_key[0] = '\0';
 		str_nz_cpy( ls[k].msg, MSG_MENU_FUNC_SET_REG_MACRO,
 				MENU_MSG_MAX_LEN );
 		ls[k].call_back = cb_menu_func_set_reg_macro;
-		ls[k].arg = (void *)i;
+		ls[k].arg = (void *)fn;
 		ls[k].flg = FLG_NULL;
 		k++;
+
+		key++;
 	}
 	ls[k].path[0] = '\0';
 }
 
 /**/
 
+bool_t	chk_executing_menu( void )
+{
+	return chk_nest_flg( g_nest_flg_executing_menu );
+}
+
+/**/
+
 char	*exec_menu( menu_t *p, menu_t *dflt, char *str_goto )
+{
+	char	*ret;
+
+	bgn_nest_flg( g_nest_flg_executing_menu );
+	ret = exec_menu_sub( p, dflt, str_goto );
+	end_nest_flg( g_nest_flg_executing_menu );
+
+	return ret;
+}
+
+/**/
+
+char	*exec_menu_sub( menu_t *p, menu_t *dflt, char *str_goto )
 {
 	bool_t	flg_short = FALSE;
 	bool_t	flg_ret = FALSE;
@@ -234,26 +334,46 @@ char	*exec_menu( menu_t *p, menu_t *dflt, char *str_goto )
 	if( p == NULL )
 		return STR_MENU_ERR;
 
+	if( !get_flg_menu_dflt() )
+		dflt = NULL;
+
 	chk_err_menu_kind( p );
 
+	/* ˥塼ΥѥȰפ鸡⡼ɽλ */
+
 	if( str_goto != NULL ){
 		flg_short = TRUE;
 		if( strcmp( p->path, str_goto ) == 0 )
 			str_goto = NULL;
 	}
 
+	/* 롦Хåؿ¹ */
+
 	if( p->call_back != NULL ){
 		menu_t	*pre_p = p;
 		char	*str_goto_call_back;
 
 		str_goto_call_back = p->call_back( &p, &dflt );
+		if( !get_flg_menu_dflt() )
+			dflt = NULL;
 
 		if( str_goto_call_back != NULL )
 			return str_goto_call_back;
 
+		/* ˥塼Υѥ񤭴ä */
+		/* ѥΥ롼ȤѤƼ¹ */
+
 		if( pre_p != p ){
+			long	len;
+
 			clr_msg();
-			return exec_menu( p, dflt, str_goto );
+
+			if( str_goto == NULL ){
+				return exec_menu( p, dflt, str_goto );
+			} else {
+				len = str_len_std( pre_p->path );
+				return exec_menu( p, dflt, &(str_goto[len]) );
+			}
 		}
 	}
 
@@ -307,6 +427,28 @@ char	*exec_menu( menu_t *p, menu_t *dflt, char *str_goto )
 	return STR_MENU_ERR;
 }
 
+/***************************************************************
+* ˥塼Υǥեȹܤ̵ͭ
+* return : ͭ?
+***************************************************************/
+
+void	set_flg_menu_dflt( bool_t flg )
+{
+	g_flg_menu_dflt = flg;
+}
+
+/**/
+
+/***************************************************************
+* ˥塼Υǥեȹܤͭ?
+* return : ͭ?
+***************************************************************/
+
+bool_t	get_flg_menu_dflt( void )
+{
+	return g_flg_menu_dflt;
+}
+
 /**/
 
 bool_t	chk_err_menu_kind( menu_t *ls )
@@ -515,23 +657,30 @@ char	*exec_menu_sel_num( long *p, long min, long max )
 	if( figure < 1 )
 		figure = 1;
 
-	sprintf( num, fmt, (int)(figure + 1), n );
-
 	sel_n = 1 + figure + MENU_NUM_OK;
 	while( 1 ){
 		long	d;
 		bool_t	flg_ret;
+		char	key;
+
+		sprintf( num, fmt, (int)(figure + 1), n );
 
-		if( g_flg_gui )
-			call_gui_num_draw( &gui_n, min, max, figure );
 		draw_menu_num( figure, sel_n, num );
 
+		key = get_key_buf_char();
+		if( key == '\0' ){
+			if( g_flg_gui ){
+				call_gui_num_draw( &gui_n, min, max,
+						figure );
+			}
+
+			key = get_key( g_flg_clr_key_buf_menu );
+		}
+
 		d = 0;
 		flg_ret = FALSE;
 
-		n = strtol( num, (char **)NULL, 10 );
-
-		switch( get_key( g_flg_clr_key_buf_menu ) ){
+		switch( key ){
 		case CASE_CANCEL:
 			call_gui_num_wipe();
 			return STR_MENU_CANCEL;
@@ -566,10 +715,25 @@ char	*exec_menu_sel_num( long *p, long min, long max )
 			break;
 		case '|':
 			n = gui_n;
+			break;
+		case KEY_MENU_NUM_OK:
 			sel_n = 1 + figure + MENU_NUM_OK;
-			flg_ret = TRUE;
+			break;
+		case KEY_MENU_NUM_CLR:
+			sel_n = 1 + figure + MENU_NUM_CLR;
+			break;
+		case KEY_MENU_NUM_MAX:
+			sel_n = 1 + figure + MENU_NUM_MAX;
+			break;
+		case KEY_MENU_NUM_MIN:
+			sel_n = 1 + figure + MENU_NUM_MIN;
+			break;
+		case KEY_MENU_NUM_SGN:
+			sel_n = 0;
 			break;
 		default:
+			if( ('A' <= key) && (key <= 'A' + figure - 1) )
+				sel_n = 1 + figure - (key - 'A') - 1;
 			break;
 		}
 
@@ -616,7 +780,7 @@ char	*exec_menu_sel_num( long *p, long min, long max )
 				n = strtol( num, (char **)NULL, 10 );
 			}
 
-			sprintf( num, fmt, (int)(figure + 1), n );
+			gui_n = n;
 		}
 	}
 
@@ -1134,8 +1298,9 @@ long	chk_key( bool_t flg_clr_key_buf, bool_t flg_chk_menu )
 {
 	long	c;
 
-	wipe_menu();
+	g_flg_chk_menu = flg_chk_menu;
 
+	wipe_menu();
 	reset_crsr();
 
 	c = get_key( flg_clr_key_buf );
@@ -1173,7 +1338,6 @@ long	chk_key( bool_t flg_clr_key_buf, bool_t flg_chk_menu )
 			wipe_menu();
 			break;
 		}
-
 		if( flg_chk_menu ){
 			if( chk_short_key( get_ls_menu_main(), c ) ){
 				wipe_menu();
@@ -1183,10 +1347,10 @@ long	chk_key( bool_t flg_clr_key_buf, bool_t flg_chk_menu )
 				wipe_menu();
 				break;
 			}
-			if( chk_short_key( get_ls_menu_func(), c ) ){
-				wipe_menu();
-				break;
-			}
+		}
+		if( chk_short_key( get_ls_menu_func(), c ) ){
+			wipe_menu();
+			break;
 		}
 
 		if( flg_chk_menu ){
@@ -1387,7 +1551,7 @@ char	*get_func( long n )
 {
 	if( n < 0 )
 		n = 0;
-	if( n >= FUNC_KEY_MAX_N )
+	if( n > FUNC_KEY_MAX )
 		n = 0;
 
 	return g_func[n];
@@ -1395,6 +1559,13 @@ char	*get_func( long n )
 
 /**/
 
+bool_t	get_flg_chk_menu( void )
+{
+	return g_flg_chk_menu;
+}
+
+/**/
+
 bool_t	get_flg_chk_short_key( void )
 {
 	return g_flg_chk_short_key;
